-----------------------------------------------
-- Company          :    tud
-- Author           :    schreiter
-- E-Mail           :    <email>
--
-- Filename         :   sc_bandpass.vhd
-- Project Name     :   OSM
-- Subproject Name  :   s_vhdl
-- Description      :   <short description>
--
-- Create Date      :   Mon Feb 20 07:03:02 2017
-- Last Change      :   $Date$
-- by               :   $Author$
-----------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.math_real.all;

--SYNOPSYS translate_off

--Uncomment the following two lines to use the declarations that are 
--provided for instantiating Xilinx primitive components
--library UNISIM;
--use UNISIM.VComponents.all;

--SYNOPSYS translate_on

entity sc_bandpass is
  generic (
    f0_rel : real := 0.1;               -- center frequency as a portion of the
                                        -- sampling frequency
    Q  : real := 10.0);                 -- Q factor
  port (
    u_i   : in  real;                   -- input voltage [Volts]
    u_o   : out real;                   -- output voltage [Volts]
    clk_i : in  std_logic);             -- sampling clock
end entity sc_bandpass;

architecture behavioral_vhdl of sc_bandpass is
  signal u1, u2, u3, u4 : real := 0.0;  -- voltages of C1, C2, C3, C4
  -- capacitances in arbitrary units, because only capacitance ratios affect
  -- the circuit
  constant C2 : real := 1.0;
  constant C1 : real := math_2_pi * f0_rel * C2;
  constant C4 : real := 1.0;
  constant C3 : real := math_2_pi * f0_rel * C4;
  constant CQ : real := C2 / Q;
begin

  -- purpose: SC step at rising and falling clock edge
  -- type   : combinational
  -- inputs : clk_i
  -- outputs: u2, u4
  SC_STEP: process (clk_i)
    variable delta_u1, delta_u2, delta_u3, delta_u4 : real;
  begin  -- process SC_STEP
    if clk_i = '1' then
      u3 <= u_i;
      delta_u3 := u_i - u3;             -- old value of u3 still valid
      delta_u4 := -C3/C4 * delta_u3;
      u4 <= u4 + delta_u4;
      u1 <= u4 + delta_u4;
      delta_u1 := u4 + delta_u4 - u1;
      delta_u2 := -C1/C2 * delta_u1 - CQ/C2 * delta_u4;
      u2 <= u2 + delta_u2;
    elsif clk_i = '0' then
      u1 <= 0.0;
      u3 <= u2;
    end if;
  end process SC_STEP;

  u_o <= u2;
            
end architecture behavioral_vhdl;
