-----------------------------------------------
-- Company          :    tud
-- Author           :    schreiter
-- E-Mail           :    <email>
--
-- Filename         :   sc_bandpass_tb.vhd
-- Project Name     :   OSM
-- Subproject Name  :   s_vhdl
-- Description      :   <short description>
--
-- Create Date      :   Mon Feb 20 10:02:02 2017
-- Last Change      :   $Date$
-- by               :   $Author$
-----------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.math_real.all;

entity sc_bandpass_tb is
end entity sc_bandpass_tb;

architecture tb_vhdl of sc_bandpass_tb is

  component sc_bandpass is
    generic (
      f0_rel : real := 0.1;               -- center frequency as a portion of the
                                          -- sampling frequency
      Q  : real := 10.0);                 -- Q factor
    port (
      u_i   : in  real;                   -- input voltage [Volts]
      u_o   : out real;                   -- output voltage [Volts]
      clk_i : in  std_logic);             -- sampling clock
  end component;
    
    constant CLKPERIODE    : time := 100 ns; --10 MHz sampling frequency
    
    signal clk        : std_logic := '0';

  signal u_i : real := 0.0;             -- SC filter input signal
  signal u_o : real;                    -- SC filter output signal

  signal pay_load : real := 0.0;               -- payload signal
  constant f_signal : real := 1.0e6;    -- 1 MHz payload signal center freq.
  constant a_signal : real := 1.0;      -- 1 Volt sinus signal amplitude  ->
                                        -- 2.0V PP
  constant t_step_ns : real := 10.0e6 / f_signal;
  
  begin
    sc_bandpass_i : sc_bandpass
      generic map (
        f0_rel => 0.1,
        Q      => 10.0)
      port map (
        u_i   => u_i,
        u_o   => u_o,
        clk_i => clk);
    

    
    clk_gen : process
    begin
        wait for CLKPERIODE/2;
        clk <= not clk;
    end process clk_gen;
    
    sig_gen : process
      variable f_sweep : real := 0.5*f_signal;
      variable phase : real := 0.0;
    begin
      pay_load <= a_signal * sin(phase); -- sin( math_2_pi * f_sweep * real( now / 1 ns)*1.0e-9);
      wait for t_step_ns * 1 ns;
      phase := phase + math_2_pi * f_sweep * t_step_ns * 1.0e-9;
      if phase > math_2_pi then
        phase := phase - math_2_pi;
      end if;
      -- one sweep in 1 ms:
      f_sweep := f_sweep + (1.5-0.5) * f_signal * t_step_ns / 1.0e6; 
      if f_sweep > 1.5*f_signal then
        f_sweep := 0.5 * f_signal;
      end if;
    end process sig_gen;

    u_i <= pay_load;
    
end architecture tb_vhdl;
